@@ -1,6 +1,6 @@
 package Google::RestApi::DriveApi3;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
@@ -8,14 +8,15 @@ use aliased 'Google::RestApi::DriveApi3::File';
 
 # TODO: switch to ReadOnly
 use constant {
-  Drive_File_Id => "[a-zA-Z0-9-_]+",
+  Drive_Endpoint => "https://www.googleapis.com/drive/v3",
+  Drive_File_Id  => "[a-zA-Z0-9-_]+",
 };
 
 sub new {
   my $class = shift;
   state $check = compile_named(
-    api      => HasMethods['api'],
-    endpoint => Str, { default => 'https://www.googleapis.com/drive/v3' },
+    api      => HasApi,
+    endpoint => Str, { default => Drive_Endpoint },
   );
   return bless $check->(@_), $class;
 }
@@ -76,6 +77,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
