@@ -1,6 +1,6 @@
 package Google::RestApi::DriveApi3::File;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
@@ -10,7 +10,7 @@ sub new {
   my $class = shift;
   my $qr_id = DriveApi3->Drive_File_Id;
   state $check = compile_named(
-    drive => HasMethods['api'],
+    drive => HasApi,
     id    => StrMatch[qr/$qr_id/],
   );
   return bless $check->(@_), $class;
@@ -33,9 +33,9 @@ sub copy {
     _extra_ => slurpy Any,
   );
   my $p = named_extra($check->(@_));
-  $p->{name} ||= $p->{title};
-  delete $p->{title};
-  $p->{content}->{name} = delete $p->{name};
+  $p->{name} //= $p->{title};
+  $p->{content}->{name} = $p->{name} if defined $p->{name};
+  delete @$p{qw(name title)};
 
   my $file_id = $self->file_id();
   $p->{uri} = "$file_id/copy";
@@ -86,6 +86,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
