@@ -1,17 +1,22 @@
 package Google::RestApi::SheetsApi4;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
+use Module::Load qw( load );
+use Try::Tiny ();
+use YAML::Any ();
+
 use aliased 'Google::RestApi::DriveApi3';
 use aliased 'Google::RestApi::SheetsApi4::Spreadsheet';
 
 # TODO: switch to ReadOnly
 use constant {
+  Sheets_Endpoint    => "https://sheets.googleapis.com/v4/spreadsheets",
   Spreadsheet_Filter => "mimeType='application/vnd.google-apps.spreadsheet'",
   Spreadsheet_Id     => DriveApi3->Drive_File_Id,
-  Spreadsheet_Uri    => "^https://docs.google.com/spreadsheets/d",
+  Spreadsheet_Uri    => "https://docs.google.com/spreadsheets/d",
   Worksheet_Id       => "[0-9]+",
   Worksheet_Uri      => "[#&]gid=([0-9]+)",
 };
@@ -20,10 +25,9 @@ sub new {
   my $class = shift;
 
   state $check = compile_named(
-    api      => HasMethods[qw(api)],
-    drive    => HasMethods[qw(filter_files)], { optional => 1 },
-    endpoint => Str, { default => 'https://sheets.googleapis.com/v4/spreadsheets' },
-    config   => HashRef, { optional => 1 },
+    api           => HasApi,
+    drive         => HasMethods[qw(filter_files)], { optional => 1 },
+    endpoint      => Str, { default => Sheets_Endpoint },
   );
   my $self = $check->(@_);
 
@@ -57,9 +61,7 @@ sub create_spreadsheet {
 
   my $result = $self->api(
     method  => 'post',
-    content => {
-      properties => $p,
-    },
+    content => { properties => $p },
   );
   for (qw(spreadsheetId spreadsheetUrl properties)) {
     $result->{$_} or LOGDIE "No '$_' returned from creating spreadsheet";
@@ -95,38 +97,39 @@ sub delete_spreadsheet {
 
 sub delete_all_spreadsheets {
   my $self = shift;
-  state $check = compile(Str);
-  my ($name) = $check->(@_);
-  my $spreadsheets = $self->spreadsheets();
-  my @spreadsheets = grep { $_->{name} eq $name; } @{ $spreadsheets->{files} };
-  DEBUG(sprintf("Deleting %d spreadsheets for name '$name'", scalar @spreadsheets));
-  $self->delete_spreadsheet($_->{id}) foreach (@spreadsheets);
-  return scalar @spreadsheets;
+
+  state $check = compile(ArrayRef->plus_coercions(Str, sub { [$_]; }));
+  my ($names) = $check->(@_);
+
+  my $count = 0;
+  foreach my $name (@$names) {
+    my @spreadsheets = grep { $_->{name} eq $name; } $self->spreadsheets();
+    $count += scalar @spreadsheets;
+    DEBUG(sprintf("Deleting %d spreadsheets for name '$name'", scalar @spreadsheets));
+    $self->delete_spreadsheet($_->{id}) foreach (@spreadsheets);
+  }
+  return $count;
 }
 
 sub spreadsheets {
   my $self = shift;
   my $drive = $self->drive();
-  return $drive->filter_files(Spreadsheet_Filter);
+  my $spreadsheets = $drive->filter_files(Spreadsheet_Filter);
+  my @spreadsheets = map { { id => $_->{id}, name => $_->{name} }; } @{ $spreadsheets->{files} };
+  return @spreadsheets;
 }
 
 sub drive {
   my $self = shift;
   if (!$self->{drive}) {
-    require Google::RestApi::DriveApi3;  # doesn't support 'aliased'
+    load DriveApi3;
     $self->{drive} = DriveApi3->new(api => $self->rest_api());
   }
   return $self->{drive};
 }
 
-sub config {
-  my $self = shift;
-  my $config = $self->{config} or return;
-  my $key = shift;
-  return defined $key ? $config->{$key} : $config;
-}
-
-sub open_spreadsheet { Spreadsheet->new(sheets => shift, @_); }
+sub open_spreadsheet { Spreadsheet->new(sheets_api => shift, @_); }
+sub transaction { shift->rest_api()->transaction(); }
 sub stats { shift->rest_api()->stats(); }
 sub rest_api { shift->{api}; }
 
@@ -145,8 +148,8 @@ Google::RestApi::SheetsApi4 - API to Google Sheets API V4.
  use aliased Google::RestApi;
  use aliased Google::RestApi::SheetsApi4;
 
- $rest = RestApi->new(%config);
- $sheets = SheetsApi4->new(api => $rest);
+ $rest_api = RestApi->new(%config);
+ $sheets_api = SheetsApi4->new(api => $rest_api);
  $sheet = $sheets->create_spreadsheet(title => 'my_name');
  $ws0 = $sheet->open_worksheet(id => 0);
 
@@ -184,7 +187,7 @@ Google::RestApi::SheetsApi4 - API to Google Sheets API V4.
  $row = $ws0->range_row("C1:1");
  $row = $ws0->range_row([<false>, 1]);
  $row = $ws0->range_row({row => 1});
- $row = $ws0->range_row([col => 3, row => 1 }, {row => 1}]);
+ $row = $ws0->range_row([{col => 3, row => 1 }, {row => 1}]);
 
  # add a header:
  $row = $ws0->range_row(1);
@@ -232,7 +235,7 @@ Google::RestApi::SheetsApi4 - API to Google Sheets API V4.
  $rows->{Id}->bold()->center();
  $rows->{Name}->red();
  # turn off fetch range and submit the formatting:
- tied(%$rows)->fetch_range()->submit_requests();
+ tied(%$rows)->fetch_range(0)->submit_requests();
 
  # iterators can be used to step through ranges:
  # a basic iterator on a column:
@@ -272,79 +275,134 @@ Google::RestApi::SheetsApi4 - API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
-SheetsApi4 is an API to Google Sheets. It is very perl-ish in that there
-is usually "more than one way to do it". It provides default behaviours
-that should be fine for most normal needs, but those behaviours can be
-overridden when necessary.
+SheetsApi4 is an API to Google Sheets. It is very perl-ish in that there is usually "more than one way to do it". It provides default behaviours
+that should be fine for most normal needs, but those behaviours can be overridden when necessary.
 
-It is assumed that you are familiar with the Google Sheets API:
-https://developers.google.com/sheets/api
+It is assumed that you are familiar with the Google Sheets API: https://developers.google.com/sheets/api
 
-The synopsis above is a quick reference. For more detailed information, see:
+C<t/tutorial/sheets/*> also has a step-by-step tutorial of creating and updating a spreadsheet, showing you the API calls and return values for each step.
 
- Google::RestApi
- Google::RestApi::SheetsApi4::Spreadsheet
- Google::RestApi::SheetsApi4::Worksheet
- 
- t/tutorial/Sheets also has a step-by-step tutorial of creating and
- updating a spreadsheet, showing you the API calls and return values
- for each step.
+=head1 NAVIGATION
+
+=over
+
+=item * L<Google::RestApi::SheetsApi4>
+
+=item * L<Google::RestApi::SheetsApi4::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Range>
+
+=item * L<Google::RestApi::SheetsApi4::Range::All>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Col>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Row>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Cell>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range>
+
+=back
 
 =head1 SUBROUTINES
 
 =over
 
-=item new(api => <Google::RestApi>);
+=item new(%args);
 
 Creates a new instance of a SheetsApi object.
 
- api: A reference to a configured RestApi instance.
+%args consists of:
 
-=item api(uri => <path_segments_string>, %args);
+=over
 
-Sets up a call to the RestApi's 'api' subroutine using the given arguments.
-The Sheets endpoint is the URI that's passed to RestApi, along with any
-other URI segment passed in the uri arg.
+=item C<api> L<<Google::RestApi>>: A reference to a configured L<Google::RestApi> instance.
 
- uri: Adds this path segment to the Sheets endpoint and calls the RestApi's
-   'api' subroutine.
- %args: Passes any extra arguments to the RestApi's 'api' subroutine (content,
-   params, method etc).
+=back
 
-You would not normally call this directly unless you were
-making a Google API call not currently supported by this API
-framework.
+=item api(%args);
 
-=item create_spreadsheet(title|name => <string>, %args);
+%args consists of:
 
-Creates a new spreadsheet with the title/name (same thing). Note that Sheets
-allows multiple spreadsheets with the same name.
+=over
 
- title|name: The title (or name) of the new spreadsheet.
- %args: Passes any extra arguments to Google Drive's create file routine.
+=item * C<uri> <path_segments_string>: Adds this path segment to the Sheets endpoint and calls the L<Google::RestApi>'s C<api> subroutine.
 
-=item copy_spreadsheet(spreadsheet_id => <string>, %args);
+=item * C<%args>: Passes any extra arguments to the L<Google::RestApi>'s C<api> subroutine (content, params, method etc).
+
+=back
+
+This is essentially a pass-through method between lower-level Worksheet/Range objects and L<Google::RestApi>, where this method adds in the Sheets endpoint.
+See <Google::RestApi::SheetsApi4::Worksheet>'s C<api> routine for how this is called. You would not normally call this directly unless you were making a Google API call not currently
+supported by this API framework.
+
+Returns the response hash from Google API.
+
+=item create_spreadsheet(%args);
+
+Creates a new spreadsheet.
+
+%args consists of:
+
+=over
+
+=item * C<title|name> <string>: The title (or name) of the new spreadsheet.
+
+=item * C<%args>: Passes through any extra arguments to Google Drive's create file routine.
+
+=back
+
+Args C<title> and C<name> are synonymous, you can use either. Note that Sheets allows multiple spreadsheets with the same name. 
+
+Normally this would be called via the Spreadsheet object, which would fill in the Drive file ID for you.
+
+Returns the object instance of the new spreadsheet object.
+
+=item copy_spreadsheet(%args);
 
 Creates a copy of a spreadsheet.
 
- spreadsheet_id: The file ID in Google Drive of the spreadsheet you want to
-   make a copy of.
- %args: Additional arguments passed to Google Drive file copy subroutine.
+%args consists of:
+
+=over
+
+=item * C<spreadsheet_id> <string>: The file ID in Google Drive of the spreadsheet you want to make a copy of.
+
+=item * C<%args>: Additional arguments passed through to Google Drive file copy subroutine.
+
+=back
+
+Returns the object instance of the new spreadsheet object.
 
 =item delete_spreadsheet(spreadsheet_id<string>);
 
-Deletes a spreadsheet from Google Drive.
+Deletes the spreadsheet from Google Drive.
 
- spreadsheet_id: The file ID in Google Drive of the spreadsheet you want to
-   delete.
+%args consists of:
+
+spreadsheet_id is the file ID in Google Drive of the spreadsheet you want to delete.
+
+Returns the Google API response.
 
 =item delete_all_spreadsheets(spreadsheet_name<string>);
 
-Deletes all spreadsheets with the given name from Google Drive. Note that
-Google Sheets allows more than one spreadsheet to have the same name. Returns
-the number of spreadsheets deleted.
+Deletes all spreadsheets with the given name from Google Drive. 
 
- spreadsheet_name: The name of the spreadsheets you want to delete.
+Returns the number of spreadsheets deleted.
 
 =item spreadsheets();
 
@@ -352,41 +410,20 @@ Returns a list of spreadsheets in Google Drive.
 
 =item drive();
 
-Returns an instance of Google Drive that shares the same RestApi as this
-SheetsApi object. You would not normally need to use this directly.
-
-=item config(key<string>);
-
-Returns a configuration item with the given key, or the entire configuration
-if no key is specified.
+Returns an instance of Google Drive that shares the same RestApi as this SheetsApi object. You would not normally need to use this directly.
 
 =item open_spreadsheet(%args);
 
-Opens a new spreadsheet from the given ID, URI, or name.
+Opens a new spreadsheet from the given id, uri, or name.
 
- %args: Passes any args to Spreadsheet->new routine.
-
-=item stats()
-
-Shows some statistics on how many get/put/post etc calls were made.
-Useful for performance tuning during development.
+%args consists of any args passed to Spreadsheet->new routine (which see).
 
 =back
 
 =head1 STATUS
 
-This api is currently very much in beta status. It is incomplete.
-There may be design flaws that need to be addressed in later
-releases. Later releases may break this release. Not all api
-calls have been implemented. Tests are not comprehensive.
-
-But it gets the job done.
-
-This is a work in progress, released at this stage in order to
-assist others, and in the hopes that others will contribute to
-its completeness.
-
-Pull requests welcome.
+This api is currently in beta status. It is incomplete. There may be design flaws that need to be addressed in later releases. Later
+releases may break this release. Not all api calls have been implemented.
 
 =head1 AUTHORS
 
@@ -400,6 +437,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
