@@ -1,23 +1,26 @@
 package Google::RestApi::SheetsApi4::Range::All;
 
-our $VERSION = '0.6.1';
+# TODO: this class has not been fully tested. use at your own risk.
+
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
+
 use parent 'Google::RestApi::SheetsApi4::Range';
 
 sub new {
   my $class = shift;
-
   state $check = compile_named(
     worksheet => HasMethods[qw(api worksheet_name)],
   );
   my $self = $check->(@_);
-
   return bless $self, $class;
 }
 
 sub range { shift->worksheet_name(); }
 
+sub is_other_inside { 1; }
+
 1;
 
 __END__
@@ -48,6 +51,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
