@@ -5,13 +5,13 @@ package Google::RestApi::SheetsApi4::RangeGroup;
 # called are commented thusly:
 # "private range routine called here!"
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
-use Carp qw(cluck confess);
-use List::Util qw(first);
-use Scalar::Util qw(looks_like_number);
+use List::Util qw( first );
+use Scalar::Util ();
+use Storable qw( dclone );
 
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Iterator';
 
@@ -19,8 +19,8 @@ sub new {
   my $class = shift;
 
   state $check = compile_named(
-    spreadsheet => HasMethods[qw(api)],
-    ranges      => ArrayRef[HasMethods['range']], { default => [] },
+    spreadsheet => HasApi,
+    ranges      => ArrayRef[HasRange], { default => [] },
   );
 
   return bless $check->(@_), $class;
@@ -28,7 +28,7 @@ sub new {
 
 sub push_ranges {
   my $self = shift;
-  state $check = compile(slurpy ArrayRef[HasMethods['range']]);
+  state $check = compile(slurpy ArrayRef[HasRange]);
   my ($ranges) = $check->(@_);
   push(@{ $self->{ranges} }, @$ranges);
   return;
@@ -37,7 +37,7 @@ sub push_ranges {
 sub clear {
   my $self = shift;
 
-  $_->clear_cached_values() foreach $self->ranges();
+  $self->clear_cached_values();
 
   my @ranges = map { $_->range(); } $self->ranges();
   my %p = (
@@ -48,6 +48,18 @@ sub clear {
   return $self->api(%p);
 }
 
+sub clear_cached_values {
+  my $self = shift;
+  $_->clear_cached_values() foreach $self->ranges();
+  return;
+}
+
+sub refresh_values {
+  my $self = shift;
+  $self->clear_cached_values();
+  return $self->values();
+}
+
 sub values {
   my $self = shift;
 
@@ -71,8 +83,8 @@ sub values {
     if ($_->dimension() =~ /^col/i) { push(@cols, $_); }
     else { push(@rows, $_); }
   }
-  $self->_batch_get('col', $p, \@cols);
-  $self->_batch_get('row', $p, \@rows);
+  $self->_batch_get('col', dclone($p), \@cols);
+  $self->_batch_get('row', dclone($p), \@rows);
 
   my @values = map { $_->values(); } @ranges;
   return \@values;
@@ -93,6 +105,7 @@ sub _batch_get {
 
   my $value_ranges = $response->{valueRanges};
   # private range routine called here!
+  my @ranges2 = map { $_->range(); } @$ranges;
   $ranges->[$_]->_cache_range_values(%{ $value_ranges->[$_] })
     foreach (0..$#$ranges);
 
@@ -186,8 +199,11 @@ sub AUTOLOAD {
 }
 
 sub ranges { @{ shift->{ranges} }; }
-sub api { shift->spreadsheet()->api(@_); }
 sub spreadsheet { shift->{spreadsheet}; }
+sub api { shift->spreadsheet()->api(@_); }
+sub sheets_api { shift->spreadsheet()->sheets_api(); }
+sub rest_api { shift->sheets_api()->rest_api(); }
+sub transaction { shift->rest_api()->transaction(); }
 
 1;
 
@@ -205,6 +221,42 @@ will submit all batch values for the underlying ranges).
 
 See the description and synopsis at Google::RestApi::SheetsApi4.
 
+=head1 NAVIGATION
+
+=over
+
+=item * L<Google::RestApi::SheetsApi4>
+
+=item * L<Google::RestApi::SheetsApi4::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Range>
+
+=item * L<Google::RestApi::SheetsApi4::Range::All>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Col>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Row>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Cell>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range>
+
+=back
+
 =head1 SUBROUTINES
 
 =over
@@ -304,6 +356,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
