@@ -1,6 +1,6 @@
 package Google::RestApi::SheetsApi4::RangeGroup::Iterator;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
@@ -10,9 +10,10 @@ sub new {
   my $class = shift;
   state $check = compile_named(
     range_group => HasMethods[qw(ranges)],
-    by          => Int->where('$_ > 0'), { default => 1 },
-    from        => Int->where('$_ > -1'), { optional => 1 },
-    to          => Int->where('$_ > -1'), { optional => 1 },
+    dim         => StrMatch[qr/^(col|row)$/], { default => 'row' },
+    by          => PositiveInt, { default => 1 },
+    from        => PositiveOrZeroInt, { optional => 1 },
+    to          => PositiveOrZeroInt, { optional => 1 },
   );
   my $self = $check->(@_);
   $self->{current} = delete $self->{from} || 0;
@@ -25,13 +26,13 @@ sub iterate {
   return if defined $self->{to} && $self->{current} + 1 > $self->{to};
 
   my @ranges = map {
-    $_->cell($self->{current});
+    $_->cell_at_offset($self->{current}, $self->{dim});
   } $self->range_group()->ranges();
-  my $range_group = $self->spreadsheet()->range_group(@ranges);
+  return if grep { undef; } @ranges;
 
-  $self->{current} += $self->{by} if $range_group;
+  $self->{current} += $self->{by};
 
-  return $range_group;
+  return $self->spreadsheet()->range_group(@ranges);
 }
 sub next { iterate(@_); }
 
@@ -52,7 +53,7 @@ A RangeGroup::Iterator is used to iterate through a range group, returning
 a range group of cells, one group at a time.
 
 Iterating over a range group assumes the range group is made up of
-a series of ranges that implement a 'cell' subroutine. This 'cell'
+a series of ranges that implement a 'cell_at_offset' subroutine. This
 routine is called on each iteration to return a Cell object that
 represents that iteration at a particular offset. The offset increases
 for each iteration.
@@ -112,6 +113,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
