@@ -1,22 +1,17 @@
 package Google::RestApi::SheetsApi4::RangeGroup::Tie;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
-use Tie::Hash;
+use Tie::Hash ();
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator';
 use parent -norequire, 'Tie::StdHash';
 
 sub iterator {
   my $self = shift;
 
-  my %ranges = map {
-    my $range = $self->ranges()->{$_};
-    $range->can('cell') ? ($_ => $range) : ();
-  } keys %{ $self->ranges() };
-  my $tied = $self->spreadsheet()->tie(%ranges);
-
+  my $tied = $self->spreadsheet()->tie(%{ $self->ranges() });
   tied(%$tied)->default_worksheet($self->worksheet())
     if $self->worksheet();
 
@@ -38,7 +33,7 @@ sub batch_values {
 sub add_ranges {
   my $self = shift;
   my %ranges = @_;
-  state $check = compile(slurpy ArrayRef[HasMethods[qw(range)]]);
+  state $check = compile(slurpy ArrayRef[HasRange]);
   $check->(CORE::values %ranges);
   @{ $self->{ranges} }{ keys %ranges } = CORE::values %ranges;
   return \%ranges;
@@ -48,17 +43,29 @@ sub add_tied {
   my $self = shift;
   state $check = compile(HashRef);
   my ($tied) = $check->(@_);
+
+  my $fetch_range = tied(%$tied)->fetch_range();
   tied(%$tied)->fetch_range(1);
-  return $self->add_ranges(%$tied);
+  $self->add_ranges(%$tied);
+  tied(%$tied)->fetch_range($fetch_range);
+
+  return $tied;
 }
 
+# fetch_range(1) turns it on, fetch_range(0) turns it off.
+# fetch_range() returns current setting. return $self so it can be
+# chained with requests.
 sub fetch_range {
   my $self = shift;
-  if (shift) {
+  my $fetch_range = shift;
+  return $self->{fetch_range} if !defined $fetch_range;
+
+  if ($fetch_range) {
     $self->{fetch_range} = 1;
   } else {
     delete $self->{fetch_range};
   }
+
   return $self;
 }
 
@@ -70,11 +77,11 @@ sub range_group {
 sub default_worksheet {
   my $self = shift;
   state $check = compile(
-    HasMethods[qw(tie_ranges tie_cells)],
+    HasMethods[qw(tie_ranges tie_cells)], { optional => 1 },
   );
   my ($worksheet) = $check->(@_);
-  $self->{worksheet} = $worksheet;
-  return;
+  $self->{worksheet} = $worksheet if $worksheet;
+  return $self->{worksheet};
 }
 
 sub TIEHASH  {
@@ -112,15 +119,8 @@ sub STORE {
   my ($key, $value) = @_;
   if (!$self->ranges()->{$key}) {
     my $worksheet = $self->worksheet()
-      or LOGDIE "No default worksheet provided for new range '$key'. Call default_worksheet() first.";
-    # have to make assumptions here. if there is a : somewhere in
-    # the range, assume it's a range, else a cell. this may not
-    # always do the right thing. in those cases, the initial tie
-    # should specify the range you want instead of auto-creating
-    # it here.
-    my $tied = $key =~ /\:/
-      ? $worksheet->tie_ranges($key)
-      : $worksheet->tie_cells($key);
+      or LOGDIE "No default worksheet provided for new range '$key'. Call default_worksheet() first.";    
+    my $tied = $worksheet->tie_ranges({ $key => $key });
     $self->add_tied($tied);
   }
 
@@ -133,9 +133,11 @@ sub STORE {
   return;
 }
 
+sub clear_cached_values { shift->range_group()->clear_cached_values(@_); }
 sub refresh_values { shift->range_group()->refresh_values(@_); }
 sub submit_values { shift->range_group()->submit_values(@_); }
 sub submit_requests { shift->range_group()->submit_requests(@_); }
+sub transaction { shift->range_group()->transaction(); }
 sub ranges { shift->{ranges}; }
 sub worksheet { shift->{worksheet}; }
 sub spreadsheet { shift->{spreadsheet}; }
@@ -242,6 +244,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
