@@ -1,12 +1,12 @@
 package Google::RestApi::SheetsApi4::Request;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
-use Hash::Merge;
-use List::MoreUtils qw(first_index);
-use Storable qw(dclone);
+use Hash::Merge ();
+use List::MoreUtils qw( first_index );
+use Storable ();
 
 sub submit_requests { LOGDIE "Pure virtual function 'submit_requests' must be overridden"; }
 
@@ -180,6 +180,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
