@@ -1,10 +1,11 @@
 package Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
 use aliased "Google::RestApi::SheetsApi4::Request";
+
 use parent "Google::RestApi::SheetsApi4::Request::Spreadsheet";
 
 sub worksheet_id { LOGDIE "Pure virtual function 'worksheet_id' must be overridden"; }
@@ -33,7 +34,7 @@ sub _freeze {
   my $self = shift;
 
   my $dim = shift;
-  state $check = compile(Int->where('$_ > -1'), { default => 0 });
+  state $check = compile(PositiveOrZeroInt, { default => 0 });
   my ($count) = $check->(@_);
 
   # "frozenColumnCount" or "frozenRowCount".
@@ -125,6 +126,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
