@@ -1,16 +1,15 @@
 package Google::RestApi::SheetsApi4::Spreadsheet;
 
-our $VERSION = '0.6.1';
+our $VERSION = '0.9.1';
 
 use Google::RestApi::Setup;
 
-use Cache::Memory::Simple;
-use Carp qw(confess);
-use Scalar::Util qw(blessed);
+use Cache::Memory::Simple ();
 
 use aliased 'Google::RestApi::SheetsApi4';
 use aliased 'Google::RestApi::SheetsApi4::Worksheet';
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup';
+use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Tie';
 
 use parent "Google::RestApi::SheetsApi4::Request::Spreadsheet";
 
@@ -20,13 +19,13 @@ sub new {
   my $qr_id = SheetsApi4->Spreadsheet_Id;
   my $qr_uri = SheetsApi4->Spreadsheet_Uri;
   state $check = compile_named(
-    sheets    => HasMethods[qw(api config spreadsheets)],
-    id        => StrMatch[qr/$qr_id/], { optional => 1 },  # https://developers.google.com/sheets/api/guides/concepts
-    name      => Str, { optional => 1 },
-    title     => Str, { optional => 1 },
-    uri       => StrMatch[qr|$qr_uri/$qr_id/|], { optional => 1 },
-    config_id => Str, { optional => 1 },
-    cache     => Int->where('$_ > -1'), { default => 5 },
+    sheets_api => HasApi,
+    # https://developers.google.com/sheets/api/guides/concepts
+    id         => StrMatch[qr/^$qr_id$/], { optional => 1 },
+    name       => Str, { optional => 1 },
+    title      => Str, { optional => 1 },
+    uri        => StrMatch[qr|^$qr_uri/$qr_id/?|], { optional => 1 },
+    cache_seconds => PositiveOrZeroNum, { default => 5 },
   );
   my $self = $check->(@_);
 
@@ -34,15 +33,6 @@ sub new {
   $self->{name} ||= $self->{title};
   delete $self->{title};
 
-  if ($self->{config_id}) {
-    my $config = $self->sheets_config($self->{config_id})
-      or LOGDIE "Config '$self->{config_id}' is missing";
-    foreach (qw(id name uri)) {
-      $self->{$_} = $config->{$_} if defined $config->{$_};
-    }
-    $self->{config} = $config->{worksheets} if $config->{worksheets};
-  }
-
   $self->{id} || $self->{name} || $self->{uri} or LOGDIE "At least one of id, name, or uri must be specified";
 
   return $self;
@@ -56,7 +46,7 @@ sub api {
   );
   my $p = named_extra($check->(@_));
   $p->{uri} = $self->spreadsheet_id() . $p->{uri};
-  return $self->sheets()->api(%$p);
+  return $self->sheets_api()->api(%$p);
 }
 
 sub spreadsheet_id {
@@ -66,16 +56,15 @@ sub spreadsheet_id {
     if ($self->{uri}) {
       my $qr_id = SheetsApi4->Spreadsheet_Id;
       my $qr_uri = SheetsApi4->Spreadsheet_Uri;
-      ($self->{id}) = $self->{uri} =~ m|$qr_uri/($qr_id)|;
+      ($self->{id}) = $self->{uri} =~ m|^$qr_uri/($qr_id)|;   # can end with '/edit'
       LOGDIE "Unable to extract a sheet id from uri" if !$self->{id};
       DEBUG("Got sheet ID '$self->{id}' via URI '$self->{uri}'.");
     } else {
-      my $spreadsheets = $self->sheets()->spreadsheets();
-      my ($spreadsheet) = grep {
-        $_->{name} eq $self->{name};
-      } @{ $spreadsheets->{files} };
-      LOGDIE "Sheet '$self->{name}' not found on google drive" if !$spreadsheet;
-      $self->{id} = $spreadsheet->{id};
+      my @spreadsheets = grep { $_->{name} eq $self->{name}; } $self->sheets_api()->spreadsheets();
+      LOGDIE "Sheet '$self->{name}' not found on Google Drive" if !@spreadsheets;
+      LOGDIE "More than one spreadsheet found with name '$self->{name}'. Specify 'id' or 'uri' instead."
+        if scalar @spreadsheets > 1;
+      $self->{id} = $spreadsheets[0]->{id};
       DEBUG("Got sheet id '$self->{id}' via spreadsheet list.");
     }
   }
@@ -83,13 +72,7 @@ sub spreadsheet_id {
   return $self->{id};
 }
 
-sub spreadsheet_uri {
-  my $self = shift;
-  $self->{uri} ||= $self->attrs('spreadsheetUrl')->{spreadsheetUrl}
-    or LOGDIE "No spreadsheet URI found from get results";
-  return $self->{uri};
-}
-
+# when 'api' is eventually called, id will be worked out if we don't already have it.
 sub spreadsheet_name {
   my $self = shift;
   $self->{name} ||= $self->properties('title')->{title}
@@ -98,6 +81,15 @@ sub spreadsheet_name {
 }
 sub spreadsheet_title { spreadsheet_name(@_); }
 
+# when 'api' is eventually called, id will be worked out if we don't already have it.
+sub spreadsheet_uri {
+  my $self = shift;
+  $self->{uri} ||= $self->attrs('spreadsheetUrl')->{spreadsheetUrl}
+    or LOGDIE "No spreadsheet URI found from get results";
+  $self->{uri} =~ s[/(edit|copy)$][]; # this isn't necessary but keeps things cleaner.
+  return $self->{uri};
+}
+
 sub attrs {
   my $self = shift;
   my $fields = shift;
@@ -115,6 +107,7 @@ sub properties {
 }
 
 # GET https://sheets.googleapis.com/v4/spreadsheets/spreadsheetId?&fields=sheets.properties
+# returns properties for each worksheet in the spreadsheet.
 sub worksheet_properties {
   my $self = shift;
   state $check = compile(Str);
@@ -140,72 +133,55 @@ sub _cache {
 
   state $check = compile(Str, CodeRef);
   my ($key, $code) = $check->(@_);
-  return $code->() if !$self->{cache};
+  return $code->() if !$self->{cache_seconds};
 
   $self->{_cache} ||= Cache::Memory::Simple->new();
+  # will run the code and store the result for x seconds.
   return $self->{_cache}->get_or_set(
-    $key, $code, $self->{cache}  # cache for x seconds
+    $key, $code, $self->{cache_seconds}
   );
 }
 
-sub cache {
+sub _cache_delete {
   my $self = shift;
-  state $check = compile(Int->where('$_ > -1'));
-  my ($cache) = $check->(@_);
-  $self->{_cache}->delete_all() if $self->{_cache};
-  delete $self->{_cache} if !$cache;
-  $self->{cache} = $cache;
+  delete $self->{_cache};
   return;
 }
 
-# each worksheet has an entry:
-# ---
-# - protectedRanges:
-#   - editors:
-#       users:
-#       - xxx@gmail.com
-#       - yyy@gmail.com
-#     protectedRangeId: 1161285259
-#     range: {}
-#     requestingUserCanEdit: !!perl/scalar:JSON::PP::Boolean 1
-#     warningOnly: !!perl/scalar:JSON::PP::Boolean 1
-# - {}
-# - {}
-# submit_requests needs to be called by the caller after this.
-sub delete_all_protected_ranges {
+# sets the number of seconds that things will be cached.
+sub cache_seconds {
   my $self = shift;
-  foreach my $worksheet (@{ $self->protected_ranges() }) {
-    my $ranges = $worksheet->{protectedRanges} or next;
-    $self->delete_protected_range($_->{protectedRangeId}) foreach (@$ranges);
-  }
-  return $self;
-}
 
-sub named_ranges {
-  my $self = shift;
-  state $check = compile(Str, { optional => 1 });
-  my ($range) = $check->(@_);
-  my $named_ranges = $self->attrs('namedRanges')->{namedRanges};
-  return $named_ranges if !$range;
-  ($range) = grep { $_->{name} eq $range; } @$named_ranges;
-  return $range;
+  state $check = compile(PositiveOrZeroNum);
+  my ($cache_seconds) = $check->(@_);
+
+  $self->{_cache}->delete_all() if $self->{_cache};
+
+  if (!$cache_seconds) {
+    $self->_cache_delete();
+    delete $self->{cache_seconds};
+  } else {
+    $self->{cache_seconds} = $cache_seconds;
+  }
+
+  return;
 }
 
 sub copy_spreadsheet {
   my $self = shift;
-  return $self->sheets()->copy_spreadsheet(
+  return $self->sheets_api()->copy_spreadsheet(
     spreadsheet_id => $self->spreadsheet_id(), @_,
   );
 }
 
 sub delete_spreadsheet {
   my $self = shift;
-  return $self->sheets()->delete_spreadsheet($self->spreadsheet_id());
+  return $self->sheets_api()->delete_spreadsheet($self->spreadsheet_id());
 }
 
 sub range_group {
   my $self = shift;
-  state $check = compile(slurpy ArrayRef[HasMethods['range']]);
+  state $check = compile(slurpy ArrayRef[HasRange]);
   my ($ranges) = $check->(@_);
   return RangeGroup->new(
     spreadsheet => $self,
@@ -216,21 +192,18 @@ sub range_group {
 sub tie {
   my $self = shift;
   my %ranges = @_;
-  tie my %tie,
-    'Google::RestApi::SheetsApi4::RangeGroup::Tie', $self;
+  tie my %tie, Tie, $self;
   tied(%tie)->add_ranges(%ranges);
   return \%tie;
 }
 
-# this is done simply to allow open_worksheet to return the
-# same worksheet instance each time it's called for the
-# same remote worksheet. this is to avoid working on multiple
-# local copies of the same remote worksheet.
-# TODO: if worksheet is renamed, registration should be
-# updated too.
+# this is done simply to allow open_worksheet to return the same worksheet instance
+# each time it's called for the same remote worksheet. this is to avoid working on
+# multiple local copies of the same remote worksheet.
+# TODO: if worksheet is renamed, registration should be updated too.
 sub _register_worksheet {
   my $self = shift;
-  state $check = compile(HasMethods['worksheet_name']);
+  state $check = compile(HasApi);
   my ($worksheet) = $check->(@_);
   my $name = $worksheet->worksheet_name();
   return $self->{registered_worksheet}->{$name} if $self->{registered_worksheet}->{$name};
@@ -238,13 +211,6 @@ sub _register_worksheet {
   return $worksheet;
 }
 
-sub config {
-  my $self = shift;
-  my $config = $self->{config} or return;
-  my $key = shift;
-  return defined $key ? $config->{$key} : $config;
-}
-
 sub submit_values {
   my $self = shift;
 
@@ -284,24 +250,26 @@ sub submit_requests {
   );
   my $p = $check->(@_);
 
+  my @all_requests = (@{ delete $p->{ranges} }, $self);   # add myself to the list.
+
   # for each object that has requests to submit, store them so that
   # they can process the responses that come back.
   my @ranges = map {
     $_->batch_requests() ? $_ : ();
-  } @{ $p->{ranges} }, $self;   # add myself to the list.
+  } @all_requests;
   return if !@ranges;
 
-  # pull out the requests hashes to be send to the rest api.
+  # pull out the requests hashes to be sent to the rest api.
   my @batch_requests = map {
     $_->batch_requests();
-  } @{ delete $p->{ranges} }, $self;
+  } @all_requests;
   return if !@batch_requests;
 
-  # call the batch requuest api. returns an array with:
-  #   $api_content (json decoded content)
-  #   $api_response (Furl::Response, also contains the string content)
-  #   $p (params of the original request)
-  # most users will only be interested in $api[0].
+  # we're about to do a bunch of requests that could affect what's in the cache.
+  # TODO: selectively invalidate the cache based on what's being submitted.
+  $self->_cache_delete();
+
+  # call the batch request api.
   $p->{content}->{requests} = \@batch_requests;
   $p->{method} = 'post';
   $p->{uri} = ':batchUpdate';
@@ -318,11 +286,65 @@ sub submit_requests {
   return $api;
 }
 
+sub named_ranges {
+  my $self = shift;
+
+  state $check = compile(RangeNamed, { optional => 1 });
+  my ($named_range_name) = $check->(@_);
+
+  my $named_ranges = $self->attrs('namedRanges')->{namedRanges};
+  return $named_ranges if !$named_range_name;
+
+  my ($named_range) = grep { $_->{name} eq $named_range_name; } @$named_ranges;
+  return $named_range;
+}
+
+sub normalize_named {
+  my $self = shift;
+
+  state $check = compile(RangeNamed);
+  my ($named_range_name) = $check->(@_);
+
+  my $named_range = $self->named_ranges($named_range_name) or return;
+  $named_range = $named_range->{range};
+  my $range = [
+    [ $named_range->{startColumnIndex} + 1, $named_range->{startRowIndex} + 1 ],
+    [ $named_range->{endColumnIndex}, $named_range->{endRowIndex} ],
+  ];
+
+  return ($named_range->{sheetId}, $range);
+}
+
 sub protected_ranges { shift->attrs('sheets.protectedRanges')->{sheets}; }
+
+# each worksheet has an entry:
+# ---
+# - protectedRanges:
+#   - editors:
+#       users:
+#       - xxx@gmail.com
+#       - yyy@gmail.com
+#     protectedRangeId: 1161285259
+#     range: {}
+#     requestingUserCanEdit: !!perl/scalar:JSON::PP::Boolean 1
+#     warningOnly: !!perl/scalar:JSON::PP::Boolean 1
+# - {}
+# - {}
+# submit_requests needs to be called by the caller after this.
+sub delete_all_protected_ranges {
+  my $self = shift;
+  foreach my $worksheet (@{ $self->protected_ranges() }) {
+    my $ranges = $worksheet->{protectedRanges} or next;
+    $self->delete_protected_range($_->{protectedRangeId}) foreach (@$ranges);
+  }
+  return $self;
+}
+
 sub open_worksheet { Worksheet->new(spreadsheet => shift, @_); }
-sub sheets_config { shift->sheets()->config(shift); }
-sub sheets { shift->{sheets}; }
-sub stats { shift->sheets()->stats(); }
+sub sheets_api { shift->{sheets_api}; }
+sub rest_api { shift->sheets_api()->rest_api(); }
+sub transaction { shift->sheets_api()->transaction(); }
+sub stats { shift->sheets_api()->stats(); }
 
 1;
 
@@ -334,42 +356,84 @@ Google::RestApi::SheetsApi4::Spreadsheet - Represents a Google Spreadsheet.
 
 =head1 DESCRIPTION
 
-See the description and synopsis at Google::RestApi::SheetsApi4.
+See the description and synopsis at L<Google::RestApi::SheetsApi4>.
+
+=head1 NAVIGATION
+
+=over
+
+=item * L<Google::RestApi::SheetsApi4>
+
+=item * L<Google::RestApi::SheetsApi4::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Range>
+
+=item * L<Google::RestApi::SheetsApi4::Range::All>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Col>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Row>
+
+=item * L<Google::RestApi::SheetsApi4::Range::Cell>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie>
+
+=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet>
+
+=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range>
+
+=back
 
 =head1 SUBROUTINES
 
 =over
 
-=item new(sheets => <SheetsApi4>, (id => <string> | name => <string> | title => <string> | uri => <string>), config_id => <string>, cache => <int>);
+=item new(%args);
 
-Creates a new instance of a Spreadsheet object. You would not normally
-call this directly, you would obtain it from the
-Sheets::open_spreadsheet routine.
+Creates a new instance of a Spreadsheet object.
 
- sheets: The parent SheetsApi4 object.
- id: The id of the spreadsheet (Google Drive file ID).
- name: The name of the spreadsheet (as shown in Google Drive).
- title: An alias for name.
- uri: The spreadsheet ID extracted from the overall URI.
- config_id: The custom config for this worksheet.
- cache: Cache information for this many seconds (default to 5, 0 disables).
+%args consists of:
 
-Only one of id/name/title/uri should be specified and this API will derive the others
-as necessary.
+=over
 
-The cache exists so that repeated calls for the same attributes
-or worksheet properties doesn't keep hammering the Google API
-over and over. The default is 5 seconds. See 'cache' below.
+=item * C<sheets_api> <SheetsApi4>: The parent SheetsApi4 object.
+
+=item * C<id> <string>: The id of the spreadsheet (Google Drive file ID).
+
+=item * C<name> <string>: The name of the spreadsheet (as shown in Google Drive).
+
+=item * C<title> <string>: An alias for name.
+
+=item * C<uri> <string>: The spreadsheet ID extracted from the overall URI.
+
+=item * C<cache_seconds> <int>: Cache information for this many seconds (default to 5, 0 disables).
+
+=back
+
+You would not normally call this directly, you would obtain it from the Sheets::open_spreadsheet routine.
+
+Only one of id/name/title/uri should be specified and this API will derive the others as necessary.
+
+The cache exists so that repeated calls for the same attributes or worksheet properties doesn't keep hammering the Google API over and over.
+The default is 5 seconds. See 'cache_seconds' below.
 
 =item api(%args);
 
-Calls the parent SheetsApi4's 'api' routine with the Sheet's
-endpoint, along with any args to be passed such as content,
-params, headers, etc.
+Calls the parent SheetsApi4's 'api' routine adding the sheet file ID, and passing through any args such as content, params, headers, etc.
 
-You would not normally call this directly unless you were
-making a Google API call not currently supported by this API
-framework.
+You would not normally call this directly unless you were making a Google API call not currently supported by this API framework.
+
+Returns the response hash from Google API.
 
 =item spreadsheet_id();
 
@@ -397,28 +461,23 @@ Returns the spreadsheet property attributes of the specified fields.
 
 =item worksheet_properties(what<string>);
 
-Returns an array ref of the properties of the worksheets
-owned by this spreadsheet.
+Returns an array ref of the properties of the worksheets owned by this spreadsheet.
 
-=item cache(<int>)
+=item cache_seconds(seconds<int>)
 
-Sets the caching time in seconds. Calling will always
-delete the existing cache. 0 also disables the cache.
+Sets the caching time in seconds. Calling will always delete the existing cache. 0 disables the cache (not recommended).
 
 =item delete_all_protected_ranges();
 
-Deletes all the protected ranges from all the worksheets
-owned by this spreadsheet.
+Deletes all the protected ranges from all the worksheets owned by this spreadsheet.
 
 =item named_ranges(name<string>);
 
-Returns the properties of the named range passed, or if
-false is passed, all the named ranges for this spreadsheet.
+Returns the properties of the named range passed, or if false is passed, all the named ranges for this spreadsheet.
 
 =item copy_spreadsheet(%args);
 
-Creates a copy of this spreadsheet and passes any args
-to the Google Drive File copy routine.
+Creates a copy of this spreadsheet and passes any args to the Google Drive file copy routine.
 
 =item delete_spreadsheet();
 
@@ -426,46 +485,54 @@ Deletes this spreadsheet from Google Drive.
 
 =item range_group(range<array>...);
 
-Creates a range group with the contained ranges.
+Creates a range group with the contained ranges. See the RangeGroup object.
 
-=item tie(ranges<hash>);
+=item tie(%args);
 
-Ties the given 'key => range' pairs into a tied range group. The
-range group can be used to send batch values (API batchUpdate) and
-batch requests (API batchRequests) as a single call once all the
-changes have been made to the overall hash.
+%args is a hash of key => Range object pairs.
 
-Turning on the 'fetch_range' property will return the underlying
-ranges on fetch so that formatting for the ranges can be set. You
-would normally only turn this on for a short time, and turn it off
-when the underlying batch requests have been submitted.
+Ties the given 'key => range' pairs into a tied range group. The range group can be used to send batch values (API batchUpdate) and
+batch requests (API batchRequests) as a single call once all the changes have been made to the overall hash.
+
+Turning on the 'fetch_range' property will return the underlying ranges on fetch so that formatting for the ranges can be set. You
+would normally only turn this on for a short time, and turn it off when the underlying batch requests have been submitted.
 
  $tied = $ss->tie(id => $range_cell);
- $tied->{id} = 1001;
- tied(%$tied)->submit_values();
-
  tied(%$tied)->fetch_range(1);
  $tied->{id}->bold()->red()->background_blue();
  tied(%$tied)->fetch_range(0)->submit_requests();
 
-See also Google::RestApi::SheetsApi4::Worksheet::tie.
+See also Worksheet::tie.
 
-=item config(key<string>)
+=item submit_values(%args);
 
-Returns the custom configuration item with the given key, or the entire
-configuration for this spreadsheet if no key is specified.
+%args consists of:
 
-=item submit_values(values<arrayref>, content<hashref>);
+=over
 
-Submits the batch values (Google API's batchUpdate) for the
-specified ranges. Content is passed to the SheetsApi4's 'api'
-call for any customized content you may need to pass.
+=item * C<ranges> <arrayref>: The ranges that have requests to be sent in one batchUpdate Google API call.
 
-=item submit_requests(requests<arrayref>, content<hashref>);
+=item * C<content> <hashref>: Any additional content to be sent to the Google API for this batch submit, for any customized content that needs to be passed.
 
-Submits any outstanding requests (Google API's batchRequests)
-for this spreadsheet. content will be passed to the SheetsApi4's
-'api' call for any customized content you may need to pass.
+=back
+
+Submits the batch values (Google API's batchUpdate) for the specified ranges. Content is passed to the SheetsApi4's 'api' call for any customized
+content you may need to pass.
+
+=item submit_requests(%args);
+
+%args consists of:
+
+=over
+
+=item * C<ranges> <arrayref>: The ranges that have requests to be sent in one batchRequest Google API call.
+
+=item * C<content> <hashref>: Any additional content to be sent to the Google API for this batch submit, for any customized content that needs to be passed.
+
+=back
+
+Submits the batch requests (Google API's batchRequest) for the specified ranges. Content is passed to the SheetsApi4's 'api' call for any customized
+content you may need to pass.
 
 =item protected_ranges();
 
@@ -473,22 +540,12 @@ Returns all the protected ranges for this spreadsheet.
 
 =item open_worksheet(%args);
 
-Creates a new Worksheet object, passing the args to that object's
-'new' routine (which see).
+Creates a new Worksheet object, passing the args to that object's 'new' routine (which see).
 
-=item sheets_config();
-
-Returns the parent SheetsApi4 config.
-
-=item sheets();
+=item sheets_api();
 
 Returns the SheetsApi4 object.
 
-=item stats()
-
-Shows some statistics on how many get/put/post etc calls were made.
-Useful for performance tuning during development.
-
 =back
 
 =head1 AUTHORS
@@ -503,6 +560,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2019, Robin Murray. All rights reserved.
+Copyright (c) 2021, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
